﻿using UnityEngine;
using System.Collections;
using IapResponse;
using IapVerifyReceipt;
using IapError;

public class IapSample : MonoBehaviour {
	#if UNITY_ANDROID
	private AndroidJavaClass unityPlayerClass = null;
	private AndroidJavaObject currentActivity = null;
	private AndroidJavaObject iapRequestAdapter = null;
    string PID;

    IEnumerator Start () 
	{
		unityPlayerClass = new AndroidJavaClass ("com.unity3d.player.UnityPlayer");
        yield return null;
		currentActivity = unityPlayerClass.GetStatic<AndroidJavaObject> ("currentActivity");
        yield return null;
        if (currentActivity != null) {
			// RequestAdapter를 초기화
			// ---------------------------------
			// 함수 parameter 정리
			// ---------------------------------
			// (1) 콜백을 받을 클래스 이름
			// (2) Activity Context
			// (3) debug 여부
		    iapRequestAdapter = new AndroidJavaObject("com.onestore.iap.unity.RequestAdapter", "IapSample", currentActivity, OneStoreIapManager.Instance.DEBUG_MODE); //Debug
		}
    }

	void Destroy () 
	{
		if (unityPlayerClass != null)
			unityPlayerClass.Dispose ();
		if (currentActivity != null)
			currentActivity.Dispose ();
		if (iapRequestAdapter != null)
			iapRequestAdapter.Dispose ();
	}

	public void RequestPurchaseHistory(string pID) 
	{
		// ---------------------------------
		// 함수 parameter 정리
		// ---------------------------------
		// (0) 메소드이름 : 구매내역 조회
		// ---------------------------------
		// (1) 필요시에는 UI 노출
		// (2) appId
		// (3) productIds
		// ----------------------------------
		iapRequestAdapter.Call ("requestPurchaseHistory", true, OneStoreIapManager.Instance.appId, pID);
	}

	public void RequestProductInfo() 
	{
		// ---------------------------------
		// 함수 parameter 정리
		// ---------------------------------
		// (0) 메소드이름 : 상품정보 조회
		// ---------------------------------
		// (1) 필요시에는 UI 노출
		// (2) appId
		// ----------------------------------
		iapRequestAdapter.Call ("requestProductInfo", true, OneStoreIapManager.Instance.appId);
	}

	public void RequestCheckPurchasability(string pID) 
	{
		// ---------------------------------
		// 함수 parameter 정리
		// ---------------------------------
		// (0) 메소드이름 : 구매가능여부 조회
		// ---------------------------------
		// (1) 필요시에는 UI 노출
		// (2) appId
		// (3) productIds
		// ----------------------------------
		iapRequestAdapter.Call ("requestCheckPurchasability", true, OneStoreIapManager.Instance.appId, pID);
	}

	public void RequestSubtractPoints(string pID) 
	{
		// ---------------------------------
		// 함수 parameter 정리
		// ---------------------------------
		// (0) 메소드이름 : 상품 속성변경 요청 
		// ---------------------------------
		// (1) 필요시에는 UI 노출
		// (2) action(아이템차감)
		// (3) appId
		// (4) productIds
		// ----------------------------------
		iapRequestAdapter.Call ("requestChangeProductProperties", true, "subtract_points", OneStoreIapManager.Instance.appId, pID);
	}

	public void RequestCancelSubscription(string pID) 
	{
		// ---------------------------------
		// 함수 parameter 정리
		// ---------------------------------
		// (0) 메소드이름 : 상품 속성변경 요청 
		// ---------------------------------
		// (1) 필요시에는 UI 노출
		// (2) action(자동결제 취소)
		// (3) appId
		// (4) productIds
		// ----------------------------------
		iapRequestAdapter.Call ("requestChangeProductProperties", true, "cancel_subscription", OneStoreIapManager.Instance.appId, pID);
	}


	//------------------------------------------------
	//
	// Command - Callback
	//
	//------------------------------------------------

	public void CommandResponse(string response) 
	{
		Debug.Log ("--------------------------------------------------------");
		Debug.Log ("[UNITY] CommandResponse >>> " + response);
		Debug.Log ("--------------------------------------------------------");

		// Parsing Json string to "Reponse" class
		Response data = JsonUtility.FromJson<Response> (response);
		Debug.Log (">>> " + data.ToString());
		Debug.Log ("--------------------------------------------------------");
	}

	public void CommandError(string message) 
	{
		Debug.Log ("--------------------------------------------------------");
		Debug.Log ("[UNITY] CommandError >>> " + message);
		Debug.Log ("--------------------------------------------------------");

		// Parsing Json string to "Error" class
		Error data = JsonUtility.FromJson<Error> (message);
		Debug.Log (">>> " + data.ToString());
		Debug.Log ("--------------------------------------------------------");
	}


	//------------------------------------------------
	//
	// Payment - Request
	//
	//------------------------------------------------

	public void RequestPaymenet(string pid)
	{
        // ---------------------------------
        // 함수 parameter 정리
        // ---------------------------------
        // (0) 메소드이름 : 구매요청
        // ---------------------------------
        // (1) appId
        // (2) productId
        // (3) proudtName
        // (4) tId
        // (5) bpInfo
        // ----------------------------------
		iapRequestAdapter.Call ("requestPayment", OneStoreIapManager.Instance.appId, pid, "", "", "");
        PID = pid;
	}

	// 결제 후 콜백 영수증을 로칼로 검증한다. (서버 검증을 한다면 생략 가능)
	public void PaymentResponse(string response) 
	{
		Debug.Log ("--------------------------------------------------------");
		Debug.Log ("[UNITY] PaymentResponse >>> " + response);
		Debug.Log ("--------------------------------------------------------");

		// Parsing Json string to "Reponse" class
		Response data = JsonUtility.FromJson<Response> (response);
		Debug.Log (">>> " + data.ToString());
		Debug.Log ("--------------------------------------------------------");
        // 로컬 검증을 호출한다. 서버에서 검증을 한다면 생략 가능
        // iapRequestAdapter.Call ("verifyReceipt", "OA00706057", data.result.TxID, data.result.SignData);
        //OneStoreIapManager.Instance.PurchaseComplete(PID, data.result.TxID, data.result.SignData);
        PID = "";
	}

	public void PaymentError(string message) 
	{
		Debug.Log ("--------------------------------------------------------");
		Debug.Log ("[UNITY] PaymentError >>> " + message);
		Debug.Log ("--------------------------------------------------------");

		// Parsing Json string to "Error" class
		Error data = JsonUtility.FromJson<Error> (message);
		Debug.Log (">>> " + data.ToString());
		Debug.Log ("--------------------------------------------------------");
        //OneStoreIapManager.Instance.PurchaseFailed(message);
        PID = "";
    }

    // 로칼 검증완료후 피드백
	public void ReceiptVerificationResponse(string result) 
	{
		Debug.Log ("--------------------------------------------------------");
		Debug.Log ("[UNITY] ReceiptVerificationResponse >>> " + result);
		Debug.Log ("--------------------------------------------------------");

		// Parsing Json string to "VerifyReceipt" class
		VerifyReceipt data = JsonUtility.FromJson<VerifyReceipt> (result);
		Debug.Log (">>> " + data.ToString());
		Debug.Log ("--------------------------------------------------------");
	}

    // 로칼 검증에서 에러가 났을때
	public void ReceiptVerificationError(string message) 
	{
		Debug.Log ("--------------------------------------------------------");
		Debug.Log ("[UNITY] ReceiptVerificationError >>> " + message);
		Debug.Log ("--------------------------------------------------------");

		// Parsing Json string to "Error" class
		Error data = JsonUtility.FromJson<Error> (message);
		Debug.Log (">>> " + data.ToString());
		Debug.Log ("--------------------------------------------------------");
	}
	#endif
}
